@@ -739,24 +739,6 @@ class CHelpItems {
 					'description' => _('Number of items in the queue which are delayed in Zabbix server or proxy by "from" till "to" seconds, inclusive.')
 				]
 			],
-			ITEM_TYPE_AGGREGATE => [
-				[
-					'key' => 'grpavg[group,key,func,<param>]',
-					'description' => _('Calculates the average value, based on the various parameters supplied. Zabbix server collects aggregate information by doing direct database queries.')
-				],
-				[
-					'key' => 'grpmin[group,key,func,<param>]',
-					'description' => _('Calculates the minimum value, based on the various parameters supplied. Zabbix server collects aggregate information by doing direct database queries.')
-				],
-				[
-					'key' => 'grpmax[group,key,func,<param>]',
-					'description' => _('Calculates the maximum value, based on the various parameters supplied. Zabbix server collects aggregate information by doing direct database queries.')
-				],
-				[
-					'key' => 'grpsum[group,key,func,<param>]',
-					'description' => _('Calculates the sum of values, based on the various parameters supplied. Zabbix server collects aggregate information by doing direct database queries.')
-				]
-			],
 			ITEM_TYPE_SIMPLE => [
 				[
 					'key' => 'icmpping[<target>,<packets>,<interval>,<size>,<timeout>]',
@@ -794,6 +776,10 @@ class CHelpItems {
 					'key' => 'vmware.cluster.status[<url>,<name>]',
 					'description' => _('VMware cluster status, <url> - VMware service URL, <name> - VMware cluster name')
 				],
+				[
+					'key' => 'vmware.cl.perfcounter[<url>,<id>,<path>,<instance>]',
+					'description' => _('VMware cluster performance counter, <url> - VMware service URL, <id> - VMware cluster id, <path> - performance counter path, <instance> - performance counter instance')
+				],
 				[
 					'key' => 'vmware.datastore.discovery[<url>]',
 					'description' => _('Discovery of VMware datastores, <url> - VMware service URL. Returns JSON')
@@ -1238,6 +1224,10 @@ class CHelpItems {
 					'key' => 'zabbix[stats,<ip>,<port>,queue,<from>,<to>]',
 					'description' => _('Number of items in the queue which are delayed in Zabbix server or proxy by "from" till "to" seconds, inclusive.')
 				],
+				[
+					'key' => 'zabbix[tcache, cache, <parameter>]',
+					'description' => _('Trend function cache statistics. Valid parameters are: all, hits, phits, misses, pmisses, items, pitems and requests.')
+				],
 				[
 					'key' => 'zabbix[trends]',
 					'description' => _('Number of values stored in table TRENDS.')
@@ -1291,15 +1281,15 @@ class CHelpItems {
 			],
 			ITEM_TYPE_JMX => [
 				[
-					'key' => 'jmx[object_name,attribute_name]',
+					'key' => 'jmx[object_name,attribute_name,<unique short description>]',
 					'description' => _('Return value of an attribute of MBean object.')
 				],
 				[
-					'key' => 'jmx.discovery[<discovery mode>,<object name>]',
+					'key' => 'jmx.discovery[<discovery mode>,<object name>,<unique short description>]',
 					'description' => _('Return a JSON array with LLD macros describing the MBean objects or their attributes. Can be used for LLD.')
 				],
 				[
-					'key' => 'jmx.get[<discovery mode>,<object name>]',
+					'key' => 'jmx.get[<discovery mode>,<object name>,<unique short description>]',
 					'description' => _('Return a JSON array with MBean objects or their attributes. Compared to jmx.discovery it does not define LLD macros. Can be used for LLD.')
 				]
 			],
